# Python imports
import os

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

# Application imports
from ..sections.sections_tab_widget import SectionsTabWidget
from ..page_body.page_widget import Page



class PagesException(Exception):
    ...


class Pages(Gtk.Notebook):
    def __init__(self, close_tab, label = None, section_manifest_pth = None):
        super(Pages, self).__init__()

        self._tab_widget  = SectionsTabWidget(self, close_tab, label)
        self._section_pth = os.path.dirname(section_manifest_pth)

        self._setup_styling()
        self._setup_signals()
        self._load_widgets()
        self._load_pages_data()


    def _setup_styling(self):
        self.set_tab_pos(1)  # NOTE:  LEFT = 0, RIGHT = 1, TOP = 2, BOTTOM = 3
        self.set_scrollable(True)

    def _setup_signals(self):
        label = self._tab_widget.get_children()[0]
        self.connect("switch-page", self._switch_page_update)
        self.connect("map", self._map_update)
        label.connect("button-release-event", self._change_section_name)

    def _load_widgets(self):
        start_box = Gtk.Box()
        end_box   = Gtk.Box()

        add_btn = Gtk.Button(label="Add Page")
        add_btn.set_image( Gtk.Image.new_from_icon_name("add", 4) )
        add_btn.set_always_show_image(True)
        add_btn.set_hexpand(True)
        add_btn.set_size_request(162, -1)
        add_btn.connect("released", self.create_page_view)

        start_box.add(add_btn)

        start_box.show_all()
        end_box.show_all()

        # PACKTYPE: 0 Start, 1 = End
        self.set_action_widget(start_box, 0)
        self.set_action_widget(end_box, 1)

    def _close_tab(self, button, page, eve = None):
        if self.get_n_pages() == 1:
            return

        page_num     = self.page_num(page)
        manifest_pth = os.path.dirname( page.get_manifest_pth() )
        msg          = f"Do you really want to delete the Section Page?"
        response     = event_system.emit_and_await("delete_files", (None, None, [manifest_pth,], msg))

        if response == -8:
            self.remove_page(page_num)


    def create_page_view(self, widget = None, eve = None, manifest_pth = None):
        page     = Page(self._close_tab, self._section_pth, manifest_pth)
        page_num = self.append_page(page, page.get_tab_widget())

        self.set_tab_detachable(page, False)
        self.set_tab_reorderable(page, True)

        page.show_all()
        self.set_current_page(page_num)

    def _switch_page_update(self, notebook, page, page_num):
        path = page.get_manifest_pth().replace("MANIFEST", "")
        settings.set_active_page(path)

    def _map_update(self, widget = None):
        path = self.get_section_pth()
        settings.set_active_section(path)

        page = self.get_nth_page( self.get_current_page() )
        path = page.get_manifest_pth()
        settings.set_active_page(path)


    def _change_section_name(self, widget = None, eve = None):
        if eve.button == 3: # NOTE: If right click
            label  = widget.get_children()[0] # NOTE: Pull from event box
            parent = self.get_parent()
            data   = [parent._manifest_pth, parent._name, parent._date]
            event_system.emit("update_section_title", (label, data))

    def _load_pages_data(self):
        if not self._section_pth:
            raise PagesException("No section path provided...")

        path  = self._section_pth
        pages = settings.clean_pages_list( os.listdir(path) )

        logger.info(f"Loading Page parts from: {path}")
        if len(pages) == 0:
            settings.generate_page_manifest(path)

        pages = settings.clean_pages_list( os.listdir(path) )
        for page in pages:
            _page = os.path.join(path, page)
            manifest = os.path.join(_page, "MANIFEST")
            self.create_page_view(None, None, manifest)

    def get_tab_widget(self):
        return self._tab_widget

    def get_section_pth(self):
        return self._section_pth
