# Python imports
import hashlib

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('GtkSource', '4')
from gi.repository import Gtk
from gi.repository import GtkSource

# Application imports
from utils.widget_save_load_controller import WidgetSaveLoadController


class TextAreaWidget(WidgetSaveLoadController,  GtkSource.View):
    def __init__(self):
        super(TextAreaWidget, self).__init__()

        self.buffer = self.get_buffer()
        self._buffer_hash = None

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        event_system.emit("register_to_query_controller", (self, self.get_query_data))
        self.show()


    def new(self):
        widget = TextAreaWidget()
        widget.buffer.set_text("Lorem ipsum dolor")
        return widget

    def delete_cleanup(self):
        ...

    def _setup_styling(self):
        ...

    def _setup_signals(self):
        self.connect("key-release-event", self._key_released)

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        ...

    def _key_released(self, widget = None, eve = None):
        if eve.type == 9:
            _hash = self.get_hash( self.get_buffer_text() )
            if _hash == self._buffer_hash:
                return

            if (eve.keyval >= 32 or eve.keyval <= 126) or (eve.keyval in [65288, 65289, 65293]):
                self.get_parent().save_needed = True
                self._buffer_hash = _hash

    def get_query_data(self):
        return self.get_buffer_text()

    def get_saveable_data(self):
        self.save_collection["data"] = self.get_buffer_text()
        self.save_collection["widget_type"] = str(type(self).__name__)
        return self.save_collection

    def set_saveable_data(self, data):
        self.save_collection = data
        self.load_saveable_data()

    def load_saveable_data(self):
        text = self.save_collection["data"]
        self._buffer_hash = self.get_hash(text)

        self.buffer.set_text(text)




    def get_buffer_text(self):
        start, end = self.buffer.get_start_iter(), self.buffer.get_end_iter()
        return self.buffer.get_slice(start, end, True)

    def get_hash(self, text):
        sha  = hashlib.sha1(text.encode("utf-8"))
        return sha.hexdigest()
