# Python imports

# Lib imports

# Application imports
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk
from gi.repository import Gdk



class DynamicHeaderWidget(Gtk.Box):
    def __init__(self, move_callback, x, y):
        super(DynamicHeaderWidget, self).__init__()

        self._move_callback = move_callback
        self._current_x     = x
        self._current_y     = y
        self._current_w, \
        self._current_h     = self.get_size_request()
        self.ctx            = None

        self._drag_start_x  = 0
        self._drag_start_y  = 0
        self._prior_time    = 0
        self._update_block  = False
        self.is_dragging    = False

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show_all()


    def _setup_styling(self):
        self.set_orientation(0)
        self.ctx = self.get_style_context()
        self.ctx.add_class("dynamic-header-widget")

    def _setup_signals(self):
        ...

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        drag_eve    = Gtk.Button(label = "...")
        hresize_eve = Gtk.Button(label = "< >")

        hresize_eve.connect("button-press-event", self._press_event)
        hresize_eve.connect("motion-notify-event", self._resize_motion_event)
        hresize_eve.connect("button-release-event", self._release_event)

        drag_eve.set_hexpand(True)
        drag_eve.connect("button-press-event", self._press_event)
        drag_eve.connect("motion-notify-event", self._motion_event)
        drag_eve.connect("button-release-event", self._release_event)
        drag_eve.connect("key-release-event", self._key_release_event)

        self.add(drag_eve)
        self.add(hresize_eve)

    def _key_release_event(self, widget = None, eve = None):
        if eve.hardware_keycode == 119: # NOTE: If Delete key
            self.get_parent().delete()


    def _press_event(self, widget = None, eve = None):
        window = settings.get_main_window()
        cursor = Gdk.Cursor(Gdk.CursorType.CROSSHAIR) if widget.get_label() == "..." else Gdk.Cursor(Gdk.CursorType.SB_H_DOUBLE_ARROW)

        window.get_window().set_cursor(cursor)
        self.is_dragging   = True
        self._drag_start_x = eve.x
        self._drag_start_y = eve.y

    def _resize_motion_event(self, widget = None, eve = None):
        if self.is_dragging:
            offset_x = eve.x - self._drag_start_x
            self._current_w += offset_x
            if self._current_w < 0:
                self._current_w = -1

            self.set_size_request(self._current_w, self._current_h)
            self.get_parent().save_needed = True

    def _motion_event(self, widget = None, eve = None):
        if self._update_block:
            self._drag_start_x = eve.x
            self._drag_start_y = eve.y
            return

        if self.is_dragging:
            offset_x = eve.x - self._drag_start_x
            offset_y = eve.y - self._drag_start_y

            self._current_x += offset_x
            self._current_y += offset_y

            self._update_block = True
            self._move_callback(self.get_parent(), self._current_x, self._current_y)
            self.get_parent().save_needed = True
            self._update_block = False

    def _release_event(self, widget = None, eve = None):
        window = settings.get_main_window()
        watch_cursor = Gdk.Cursor(Gdk.CursorType.ARROW)
        window.get_window().set_cursor(watch_cursor)

        self.is_dragging   = False
        self._drag_start_x = 0
        self._drag_start_y = 0
